import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * The bloodstream is the setting for our White Blood Cell scenario. 
 * It's a place where blood cells, bacteria and viruses float around.
 * 
 * 5.1: The WBC can move up and down; bacteria continually appear and flow from right to left. Nothing else.
 * 
 * 5.2: The bloodstream is the world; it has an inner vein and a lining which is basically the wall of the vein.
 * 
 * @author Michael Kölling
 * @version 0.1
 */
public class Bloodstream extends World
{
    private int score; // 5.38
    private int time; // 5.51
    
    private int plaqueProbability;
    static public int plaqueCount;
    static public int complement;
    
    /**
     * Constructor: Set up the starting objects.
     */
    public Bloodstream()
    {    
        super(780, 360, 1); 
        prepare();
        setPaintOrder(Border.class, WhiteCell.class, Plaque.class, Lining.class); // 5.28
        // score = 0; 5.39
        score = 30;
        time = 2000; // 5.52
        showScore(); // 5.47
        showTime(); // 5.53
        
        plaqueProbability = 4;
        plaqueCount = 0;
        complement = 360 - plaqueCount;
    }

    /**
     * Create new floating objects at irregular intervals.
     */
    public void act()
    {
        if (Greenfoot.getRandomNumber(100) < 3)
        {
            if(plaqueCount > 70)
            {
                addObject(new Bacteria(), 779, Greenfoot.getRandomNumber(220) + 70);
            }

            else
            {
                addObject(new Bacteria(), 779, Greenfoot.getRandomNumber(360 - 2*plaqueCount) + plaqueCount);
            }
        }
        
        // 5.8 
        // No longer relavent; see the Lining class.
        
        // 5.10
        if(Greenfoot.getRandomNumber(100) < 1)
        {
            if(plaqueCount > 70)
            {
                addObject(new Virus(), 779, Greenfoot.getRandomNumber(220) + 70);
            }

            else
            {
                addObject(new Virus(), 779, Greenfoot.getRandomNumber(360 - 2*plaqueCount) + plaqueCount);
            }
        }
        
        // 5.23
        if(Greenfoot.getRandomNumber(100) < 6)
        {
            if(plaqueCount > 70)
            {
                addObject(new RedCell(), 779, Greenfoot.getRandomNumber(220) + 70);
            }

            else
            {
                addObject(new RedCell(), 779, Greenfoot.getRandomNumber(360 - 2*plaqueCount) + plaqueCount);
            }
        }
        
        if(Greenfoot.getRandomNumber(1500) < plaqueProbability)
        {
            addObject(new Plaque(), Greenfoot.getRandomNumber(520) + 130, plaqueCount + 10); 
            plaqueProbability++;
            plaqueCount = plaqueCount + 2;
        }
        
        if(Greenfoot.getRandomNumber(1500) + plaqueProbability > 1499)
        {
            addObject(new Plaque(), Greenfoot.getRandomNumber(520) + 130, 350 - plaqueCount);   
            plaqueProbability++;
            plaqueCount = plaqueCount + 2;
        }
        
        countTime(); // 5.54    
    }
    
    /** 5.41
     * A method which increases our score whenever a WBC touches a bacterium. (See the WhiteCell class for more details.)
     */
    
    // 12-15-22 Class: Abstration has been implemented!
    
    // 5.43
    public void addScore(int points)
    {
        score = score + points;
        // showText("Score: " + score, 80, 25);
        showScore(); // 5.46
        
        if(score < 0)
        {
            showText("Sorry, you lost!", 380, 90);
            Greenfoot.playSound("game-over.wav");// 5.50
            Greenfoot.stop();
        }
    }
    
    // 5.46
    private void showScore()
    {
        showText("Score: " + score, 60, 25); 
    }
    
    // 5.54
    private void countTime()
    {
        time--;
        showTime();
        
        if(time == 0)
        {
            showEndMessage(); // 5.56
            Greenfoot.stop(); // 5.55
        }
    }
    
    // 5.53
    private void showTime()
    {
        showText("Time Left: " + time, 700, 25);
    }

    // 5.56
    private void showEndMessage()
    {
        showText("Time is up; you've won! Final score: "+ score, 390, 270);    
    }
    
    /**
     * Prepare the world for the start of the program. In this case: Create
     * a white blood cell and the lining at the edge of the blood stream.
     */
    private void prepare()
    {
        WhiteCell whitecell = new WhiteCell();
        addObject(whitecell, 95, 179); // 5.27

        Lining lining1 = new Lining();
        addObject(lining1, 50, 1);
        Lining lining2 = new Lining();
        addObject(lining2, 215, 1);
        Lining lining3 = new Lining();
        addObject(lining3, 380, 1);
        Lining lining4 = new Lining();
        addObject(lining4, 545, 1);
        Lining lining5 = new Lining();
        addObject(lining5, 710, 1);
        
        Lining lining6 = new Lining();
        addObject(lining6, 50, 359);
        Lining lining7 = new Lining();
        addObject(lining7, 215, 359);
        Lining lining8 = new Lining();
        addObject(lining8, 380, 359);
        Lining lining9 = new Lining();
        addObject(lining9, 545, 359);
        Lining lining10 = new Lining();
        addObject(lining10, 710, 359);     
    
        // 5.26 - The paint order is still not correct, and the WBC isn't completely visible.
        Border border1 = new Border();
        addObject(border1, 0, 180);
        Border border2 = new Border();
        addObject(border2, 770, 180);
        // Crazy Control, Possibly With Mouse: border2.setLocation(737, 220);
    }
    
    // 5.3: I'm not really sure how the author came up with the shape for the lining. It may be periodic.
}